@@ -49,7 +49,7 @@
  *  prior written authorization.
  */
 
-#include <config.h>
+#include "config.h"
 
 #include <Carbon/Carbon.h>
 #include <AppKit/NSEvent.h>
@@ -61,7 +61,16 @@
 
 static GdkKeymap *default_keymap = NULL;
 
+/* Note: we could check only if building against the 10.5 SDK instead, but
+ * that would make non-xml layouts not work in 32-bit which would be a quite
+ * bad regression. This way, old unsupported layouts will just not work in
+ * 64-bit.
+ */
+#ifdef __LP64__
+static TISInputSourceRef current_layout = NULL;
+#else
 static KeyboardLayoutRef current_layout = NULL;
+#endif
 
 /* This is a table of all keyvals. Each keycode gets KEYVALS_PER_KEYCODE entries.
  * TThere is 1 keyval per modifier (Nothing, Shift, Alt, Shift+Alt);
@@ -128,7 +137,8 @@ const static struct {
   { 111, GDK_F12, 0 },
   { 105, GDK_F13, 0 },
   { 107, GDK_F14, 0 },
-  { 113, GDK_F15, 0 }
+  { 113, GDK_F15, 0 },
+  { 106, GDK_F16, 0 }
 };
 
 const static struct {
@@ -178,28 +188,46 @@ const static struct {
 static void
 maybe_update_keymap (void)
 {
+  const void *chr_data = NULL;
+
+#ifdef __LP64__
+  TISInputSourceRef new_layout = TISCopyCurrentKeyboardLayoutInputSource ();
+  CFDataRef layout_data_ref;
+
+#else
   KeyboardLayoutRef new_layout;
+  KeyboardLayoutKind layout_kind;
 
   KLGetCurrentKeyboardLayout (&new_layout);
+#endif
 
   if (new_layout != current_layout)
     {
       guint *p;
       int i;
 
-      KeyboardLayoutKind layout_kind;
-      
       g_free (keyval_array);
       keyval_array = g_new0 (guint, NUM_KEYCODES * KEYVALS_PER_KEYCODE);
 
+#ifdef __LP64__
+      layout_data_ref = (CFDataRef) TISGetInputSourceProperty
+	(new_layout, kTISPropertyUnicodeKeyLayoutData);
+
+      if (layout_data_ref)
+	chr_data = CFDataGetBytePtr (layout_data_ref);
+
+      if (chr_data == NULL)
+	{
+	  g_error ("cannot get keyboard layout data");
+	  return;
+	}
+#else
       /* Get the layout kind */
       KLGetKeyboardLayoutProperty (new_layout, kKLKind, (const void **)&layout_kind);
 
       /* 8-bit-only keyabord layout */
       if (layout_kind == kKLKCHRKind)
 	{ 
-	  const void *chr_data;
-	  
 	  /* Get chr data */
 	  KLGetKeyboardLayoutProperty (new_layout, kKLKCHRData, (const void **)&chr_data);
 	  
@@ -255,7 +283,15 @@ maybe_update_keymap (void)
 			p[j] = GDK_ISO_Left_Tab;
 
 		      if (!found)
-			p[j] = gdk_unicode_to_keyval (uc);
+                        {
+                          guint tmp;
+                          
+                          tmp = gdk_unicode_to_keyval (uc);
+                          if (tmp != (uc | 0x01000000))
+                            p[j] = tmp;
+                          else
+                            p[j] = 0;
+                        }
 		    }
 		}
 	      
@@ -273,10 +309,9 @@ maybe_update_keymap (void)
       /* unicode keyboard layout */
       else if (layout_kind == kKLKCHRuchrKind || layout_kind == kKLuchrKind)
 	{ 
-	  const void *chr_data;
-	  
 	  /* Get chr data */
 	  KLGetKeyboardLayoutProperty (new_layout, kKLuchrData, (const void **)&chr_data);
+#endif
 	  
 	  for (i = 0; i < NUM_KEYCODES; i++) 
 	    {
@@ -322,9 +357,23 @@ maybe_update_keymap (void)
 			      break;
 			    }
 			}
+
+		      /* Special-case shift-tab since GTK+ expects
+		       * GDK_ISO_Left_Tab for that.
+		       */
+		      if (found && p[j] == GDK_Tab && modifiers[j] == shiftKey)
+			p[j] = GDK_ISO_Left_Tab;
 		      
 		      if (!found)
-			p[j] = gdk_unicode_to_keyval (uc);
+                        {
+                          guint tmp;
+                          
+                          tmp = gdk_unicode_to_keyval (uc);
+                          if (tmp != (uc | 0x01000000))
+                            p[j] = tmp;
+                          else
+                            p[j] = 0;
+                        }
 		    }
 		}
 	      
@@ -338,12 +387,14 @@ maybe_update_keymap (void)
 		  p[1] == p[3])
 		p[2] = p[3] = 0;
 	    }
+#ifndef __LP64__
 	}
       else
 	{
 	  g_error ("unknown type of keyboard layout (neither KCHR nor uchr)"
 	           " - not supported right now");
 	}
+#endif
 
       for (i = 0; i < G_N_ELEMENTS (known_keys); i++)
 	{
@@ -358,8 +409,8 @@ maybe_update_keymap (void)
 	{
 	  p = keyval_array + known_numeric_keys[i].keycode * KEYVALS_PER_KEYCODE;
 
-	  if (p[0] == known_numeric_keys[i].normal_keyval);
-	      p[0] = known_numeric_keys[i].keypad_keyval;
+	  if (p[0] == known_numeric_keys[i].normal_keyval)
+            p[0] = known_numeric_keys[i].keypad_keyval;
 	}
       
       if (current_layout)
@@ -386,6 +437,20 @@ gdk_keymap_get_direction (GdkKeymap *keymap)
   return PANGO_DIRECTION_NEUTRAL;
 }
 
+gboolean
+gdk_keymap_have_bidi_layouts (GdkKeymap *keymap)
+{
+  /* FIXME: Can we implement this? */
+  return FALSE;
+}
+
+gboolean
+gdk_keymap_get_caps_lock_state (GdkKeymap *keymap)
+{
+  /* FIXME: Implement this. */
+  return FALSE;
+}
+
 gboolean
 gdk_keymap_get_entries_for_keyval (GdkKeymap     *keymap,
                                    guint          keyval,
@@ -530,8 +595,8 @@ translate_keysym (guint           hardware_keycode,
   if (state & GDK_LOCK_MASK)
     {
       guint upper = gdk_keyval_to_upper (tmp_keyval);
-	if (upper != tmp_keyval)
-	  tmp_keyval = upper;
+      if (upper != tmp_keyval)
+        tmp_keyval = upper;
     }
 
   return tmp_keyval;
@@ -587,6 +652,14 @@ gdk_keymap_translate_keyboard_state (GdkKeymap       *keymap,
   return TRUE;
 }
 
+gboolean
+gdk_keymap_map_virtual_modifiers (GdkKeymap       *keymap,
+                                  GdkModifierType *state)
+{
+  /* FIXME: For now, we've mimiced the Windows backend. */
+  return TRUE;
+}
+
 /* What sort of key event is this? Returns one of
  * GDK_KEY_PRESS, GDK_KEY_RELEASE, GDK_NOTHING (should be ignored)
  */
